(* Bearbeiten der Versionsinformation einer Projektgruppe
   ======================================================
   Erstellt fr jedes Projekt der Gruppe je eine Ressourcendatei mit den
   Versionsangaben fr die einzelnen Plattformen (z.Zt. Win32 und Win64) auf
   Basis der Vorlage Version.trc.
   Die so erstellten rc-Dateien werden automatisch in res-Dateien compiliert.
   Die Einstellungen fr jede Projektgruppe werden in einer Datei
   <Projektname>.vprj im ini-Format gespeichert.

    2021, Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1.0:  April 2021
   last modified: October 2023
   *)

unit EditVersResMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Buttons, Vcl.StdCtrls, Vcl.ComCtrls,
  Vcl.ExtCtrls, Vcl.CheckLst, Vcl.Menus, SynEdit;

const
  ProgName = 'Project Version Resource Editor';
  Vers = ' - Vers. 1.0';
  CopRgt = ' 2021-2023 Dr. J. Rathlev, D-24222 Schwentinental';
  EMailAdr = 'kontakt(a)rathlev-home.de';

resourcestring
  rsFileVersionMajor = 'Major version';
  rsFileVersionMinor = 'Minor version';
  rsFileVersionRelease = 'Release';
  rsFileVersionBuild = 'Build';
  rsFileDescription = 'Description';
  rsProductVersionMajor = 'Major product version';
  rsProductVersionMinor = 'Minor product version';
  rsAuthor = 'Author';
  rsCompanyName = 'Company';
  rsLegalCopyright = 'Copyright';
  rsProductName = 'Product name';
  rsPlatform = 'Platform';

const
  NumPlaceholders = 12;

type
  TProjectState = (psEnabled,psWin32,psWin64);
  TProjectStates = set of TProjectState;
  TPlaceholder = record
    rp : pointer;
    ph : string;
    end;
  TPlaceholders = array[0..NumPlaceholders-1] of TPlaceholder;

  TProjectData = record
    FileVersionMajor,
    FileVersionMinor,
    FileVersionRelease,
    FileVersionBuild : integer;
    FileDescription  : string;
    Status           : TProjectStates;
    end;

  TProjectInfo = class (TObject)
    ProjectData : TProjectData;
    constructor Create (const AProjectData : TProjectData);
    end;

  TGroupData = record
    ProductVersionMajor,
    ProductVersionMinor : integer;
    CompanyName,
    LegalCopyright,
    ProductName,
    PlatformType,
    Author              : string;
    end;

  TMainForm = class(TForm)
    lbProjectName: TLabel;
    cbProjectGroup: TComboBox;
    OpenDialog: TOpenDialog;
    Label1: TLabel;
    gbGroup: TGroupBox;
    gbProject: TGroupBox;
    edProdMajor: TLabeledEdit;
    udProdMajor: TUpDown;
    edProdMinor: TLabeledEdit;
    udProdMinor: TUpDown;
    edCopyright: TLabeledEdit;
    edAuthor: TLabeledEdit;
    edCompany: TLabeledEdit;
    edProduct: TLabeledEdit;
    edMajor: TLabeledEdit;
    udMajor: TUpDown;
    udMinor: TUpDown;
    edMinor: TLabeledEdit;
    udRelease: TUpDown;
    edRelease: TLabeledEdit;
    edBuild: TLabeledEdit;
    udBuild: TUpDown;
    edProject: TLabeledEdit;
    btProcess: TBitBtn;
    lbProjects: TCheckListBox;
    Label2: TLabel;
    cbWin32: TCheckBox;
    cbWin64: TCheckBox;
    paProcess: TPanel;
    mmDos: TMemo;
    pcMain: TPageControl;
    tsProcess: TTabSheet;
    tsSettings: TTabSheet;
    paSettings: TPanel;
    edTemplate: TLabeledEdit;
    edResPath: TLabeledEdit;
    edSuffix32: TLabeledEdit;
    edSuffix64: TLabeledEdit;
    edResCompiler: TLabeledEdit;
    btProjectGroup: TBitBtn;
    btTemplate: TBitBtn;
    btResCompiler: TBitBtn;
    btSettings: TBitBtn;
    btBack: TBitBtn;
    btInfo: TBitBtn;
    btQuit: TBitBtn;
    btEdit: TBitBtn;
    Label3: TLabel;
    tsEdit: TTabSheet;
    paEditTop: TPanel;
    Label4: TLabel;
    laTemplateFile: TLabel;
    paButtons: TPanel;
    btOk: TBitBtn;
    btCancel: TBitBtn;
    seTemplate: TSynEdit;
    sbCheckAll: TSpeedButton;
    sbCheckNone: TSpeedButton;
    cbPlaceholders: TComboBox;
    Label5: TLabel;
    btnInsert: TBitBtn;
    Label6: TLabel;
    edDefAuthor: TLabeledEdit;
    edDefCompany: TLabeledEdit;
    edDefCopyright: TLabeledEdit;
    pmProjects: TPopupMenu;
    pmiEditList: TMenuItem;
    N1: TMenuItem;
    pmiCancel: TMenuItem;
    sbCopyMajor: TSpeedButton;
    sbCopyMinor: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbProjectGroupCloseUp(Sender: TObject);
    procedure lbProjectsClick(Sender: TObject);
    procedure lbProjectsClickCheck(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btProcessClick(Sender: TObject);
    procedure btProjectGroupClick(Sender: TObject);
    procedure btResCompilerClick(Sender: TObject);
    procedure btTemplateClick(Sender: TObject);
    procedure btSettingsClick(Sender: TObject);
    procedure btBackClick(Sender: TObject);
    procedure btQuitClick(Sender: TObject);
    procedure btInfoClick(Sender: TObject);
    procedure btEditClick(Sender: TObject);
    procedure btCancelClick(Sender: TObject);
    procedure btOkClick(Sender: TObject);
    procedure sbCheckAllClick(Sender: TObject);
    procedure sbCheckNoneClick(Sender: TObject);
    procedure btnInsertClick(Sender: TObject);
    procedure seTemplateStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure pmiEditListClick(Sender: TObject);
    procedure sbCopyMajorClick(Sender: TObject);
    procedure sbCopyMinorClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgName,ProgVers,
    ProgVersDate,
    AppPath,ProgPath,
    UserPath,
    Ininame,LastPath,
    LastGroupFile,
    LastProjectGrp   : string;
    Template         : TStringList;
    DefHeight,
    LastIndex        : integer;
    function SelectTemplate : boolean;
    function SelectCompiler : boolean;
    function SelectProjectGroup : boolean;
    procedure ReadGlobalOptions;
    procedure ReadProjectOptions (ProjIndex : integer; var pd : TProjectData);
    procedure ReadData (const GrpDatName : string);
    procedure WriteData (const GrpDatName : string);
    function ReadGroupFile (const GrpName : string) : boolean;
    procedure ShowProjectData;
    procedure SaveProjectData;
    procedure Compile (const APath,RPath,FileName : string);
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses System.IniFiles, System.StrUtils, GnuGetText, InitProg, PathUtils,
  WinUtils, MsgDialogs, NumberUtils, StringUtils, EditHistListDlg;

constructor TProjectInfo.Create (const AProjectData : TProjectData);
begin
  inherited Create;
  ProjectData:=AProjectData;
  end;

const
  IniExt = 'ini';
  DatExt = 'vprj';
  RcExt  = 'rc';
  ResExt = 'res';
  TemplExt = 'trc';

  defComp = 'brcc32.exe';
  defResPath = 'Resources';
  defTemplate = 'VersInfo.trc';
  defW32Suffix = '-32';
  defW64Suffix = '-64';

  sKeyProject = '<Projects Include=';

  CfgSekt = 'Config';

  iniLastPath = 'LastPath';
  iniLastProjGrp = 'LastProjGroup';
  iniProjGrpCnt  = 'ProjGroupCount';
  iniProjectGroups = 'ProjGroups';
  iniTemplate = 'TemplateName';
  iniResPath = 'ResourcePath';
  iniResComp = 'ResourceCompiler';
  iniWin32 = 'Win32Suffix';
  iniWin64 = 'Win64Suffix';
  iniDefAuthor = 'DefaultAuthor';
  iniDefComp = 'DefaultCompany';
  iniDefCopRgt = 'DefaultCopyright';

  prjGlobal = 'Global';

  sFileVersionMajor = 'fvMajor';
  sFileVersionMinor = 'fvMinor';
  sFileVersionRelease = 'fvRelease';
  sFileVersionBuild = 'fvBuild';
  sFileDescription = 'Description';
  sProductVersionMajor = 'pvMajor';
  sProductVersionMinor = 'pvMinor';
  sAuthor = 'Author';
  sCompanyName = 'Company';
  sLegalCopyright = 'Copyright';
  sProductName = 'Product';
  sPlatform = 'Platform';
  sStatus = 'Status';

  sWin32 = '32-bit';
  sWin64 = '64-bit';
  AllStates : TProjectStates = [psEnabled,psWin32,psWin64];

var
  Placeholders : TPlaceholders =
    ((rp : @rsFileVersionMajor; ph : sFileVersionMajor),
     (rp : @rsFileVersionMinor; ph : sFileVersionMinor),
     (rp : @rsFileVersionRelease; ph : sFileVersionRelease),
     (rp : @rsFileVersionBuild ; ph : sFileVersionBuild ),
     (rp : @rsFileDescription; ph : sFileDescription),
     (rp : @rsProductVersionMajor; ph : sProductVersionMajor),
     (rp : @rsProductVersionMinor; ph : sProductVersionMinor),
     (rp : @rsAuthor; ph : sAuthor),
     (rp : @rsCompanyName; ph : sCompanyName),
     (rp : @rsLegalCopyright; ph : sLegalCopyright),
     (rp : @rsProductName; ph : sProductName),
     (rp : @rsPlatform; ph : sPlatform));

procedure TMainForm.FormCreate(Sender: TObject);
var
  i,n : integer;
  s   : string;
begin
  TranslateComponent (self);
  InitPaths (AppPath,UserPath,ProgPath);
  InitVersion (ProgName,Vers,CopRgt,3,3,ProgName,ProgVers,ProgVersDate);
  IniName:=PrgName;
  IniName:=Erweiter(AppPath,IniName,IniExt);
  with TIniFile.Create(IniName) do begin
    LastPath:=ReadString(CfgSekt,iniLastPath,UserPath);
    s:=ReadString(CfgSekt,iniTemplate,defTemplate);
    edTemplate.Text:=ExpandRelativePath(PrgPath,s);
    edResCompiler.Text:=ReadString(CfgSekt,iniResComp,'');
    edResPath.Text:=ReadString(CfgSekt,iniResPath,defResPath);
    edSuffix32.Text:=ReadString(CfgSekt,iniWin32,defW32Suffix);
    edSuffix64.Text:=ReadString(CfgSekt,iniWin64,defW64Suffix);
    edDefAuthor.Text:=ReadString(CfgSekt,iniDefAuthor,'');
    edDefCompany.Text:=ReadString(CfgSekt,iniDefComp,'');
    edDefCopyright.Text:=ReadString(CfgSekt,iniDefCopRgt,'');
    LastProjectGrp:=ReadString(CfgSekt,iniLastProjGrp,'');
    n:=ReadInteger(CfgSekt,iniProjGrpCnt,0);
    for i:=0 to n-1 do begin
      s:=ReadString(CfgSekt,iniProjectGroups+ZStrint(i+1,2),'');
      if FileExists(s) then cbProjectGroup.Items.Add(s);
      end;
    Free;
    end;
  Caption:=_('Edit Project Version Resources')+Space+ProgVers;
  Template:=TStringList.Create;
  LastIndex:=-1; LastGroupFile:='';
  DefHeight:=ClientHeight;
  pcMain.ActivePageIndex:=0;
  for i:=0 to NumPlaceholders-1 do begin
    s:=LoadResString(Placeholders[i].rp);
    cbPlaceholders.AddItem(s,pointer(i));
    end;
  cbPlaceholders.ItemIndex:=0;
  end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  i : integer;
begin
  with TIniFile.Create(IniName) do begin
    WriteString(CfgSekt,iniLastPath,LastPath);
    WriteString(CfgSekt,iniTemplate,MakeRelativePath(PrgPath,edTemplate.Text));
    WriteString(CfgSekt,iniResComp,edResCompiler.Text);
    WriteString(CfgSekt,iniResPath,edResPath.Text);
    WriteString(CfgSekt,iniWin32,edSuffix32.Text);
    WriteString(CfgSekt,iniWin64,edSuffix64.Text);
    WriteString(CfgSekt,iniDefAuthor,edDefAuthor.Text);
    WriteString(CfgSekt,iniDefComp,edDefCompany.Text);
    WriteString(CfgSekt,iniDefCopRgt,edDefCopyright.Text);
    WriteString(CfgSekt,iniLastProjGrp,LastProjectGrp);
    with cbProjectGroup.Items do begin
      WriteInteger(CfgSekt,iniProjGrpCnt,Count);
      for i:=0 to Count-1 do begin
        WriteString(CfgSekt,iniProjectGroups+ZStrint(i+1,2),Strings[i]);
        end;
      end;
    Free;
    end;
  Template.Free;
  end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if (length(edTemplate.Text)=0) or not FileExists(edTemplate.Text) then begin
    if not SelectTemplate then begin
      Close; Exit;
      end
    end;
  Template.LoadFromFile(edTemplate.Text);
  if (length(edResCompiler.Text)=0) or not FileExists(edResCompiler.Text) then begin
    if not SelectCompiler then begin
      Close; Exit;
      end;
    end;
  if (length(LastProjectGrp)=0) or not FileExists(LastProjectGrp) then begin
    if not SelectProjectGroup then begin
      Close; Exit;
      end;
    end
  else ReadGroupFile(LastProjectGrp);
  end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WriteData(LastGroupFile);
  end;

procedure TMainForm.ReadData (const GrpDatName : string);
var
  i       : integer;
  w       : word;
  pd      : TProjectData;
begin
  LastGroupFile:=GrpDatName;
  with TIniFile.Create(GrpDatName) do begin
    if SectionExists(prjGlobal) then begin
      udProdMajor.Position:=ReadInteger(prjGlobal,sProductVersionMajor,1);
      udProdMinor.Position:=ReadInteger(prjGlobal,sProductVersionMinor,0);
      edCompany.Text:=ReadString(prjGlobal,sCompanyName,'');
      edCopyright.Text:=ReadString(prjGlobal,sLegalCopyright,'');
      edProduct.Text:=ReadString(prjGlobal,sProductName,'');
      edAuthor.Text:=ReadString(prjGlobal,sAuthor,'');
      end
    else begin  // Basisdaten erstmalig einlesen
      udProdMajor.Position:=1; udProdMinor.Position:=0;
      edAuthor.Text:=''; edCompany.Text:='';
      edCopyright.Text:=''; edProduct.Text:='';
      ReadGlobalOptions;
      with edAuthor do if length(Text)=0 then Text:=edDefAuthor.Text;
      with edCompany do if length(Text)=0 then Text:=edDefCompany.Text;
      with edCopyright do if length(Text)=0 then Text:=edDefCopyright.Text;
      end;
    with lbProjects do begin
      for i:=0 to Count-1 do begin
        if SectionExists(Items[i]) then with pd do begin
          FileVersionMajor:=ReadInteger(Items[i],sFileVersionMajor,1);
          FileVersionMinor:=ReadInteger(Items[i],sFileVersionMinor,0);
          FileVersionRelease:=ReadInteger(Items[i],sFileVersionRelease,0);
          FileVersionBuild:=ReadInteger(Items[i],sFileVersionBuild,0);
          FileDescription:=ReadString(Items[i],sFileDescription,'');
          w:=ReadInteger(Items[i],sStatus,SetToWord(@AllStates));
          WordToSet(w,@Status);
          end
        else begin  // neues Projekt
          with pd do begin  // Vorgabewerte
            FileVersionMajor:=1; FileVersionMinor:=0;
            FileVersionRelease:=0; FileVersionBuild:=0;
            FileDescription:=''; Status:=AllStates;
            end;
          ReadProjectOptions(i,pd);
          end;
        Checked[i]:=psEnabled in pd.Status;
        Items.Objects[i]:=TProjectInfo.Create(pd);
        end;
      end;
    Free;
    end;
  end;

procedure TMainForm.WriteData (const GrpDatName : string);
var
  i       : integer;
begin
  if length(GrpDatName)>0 then begin
    with TIniFile.Create(GrpDatName) do begin
      WriteInteger(prjGlobal,sProductVersionMajor,udProdMajor.Position);
      WriteInteger(prjGlobal,sProductVersionMinor,udProdMinor.Position);
      WriteString(prjGlobal,sCompanyName,edCompany.Text);
      WriteString(prjGlobal,sLegalCopyright,edCopyright.Text);
      WriteString(prjGlobal,sProductName,edProduct.Text);
      WriteString(prjGlobal,sAuthor,edAuthor.Text);
      with lbProjects do for i:=0 to Count-1 do begin
        with (Items.Objects[i] as TProjectInfo).ProjectData do begin
          WriteInteger(Items[i],sFileVersionMajor,FileVersionMajor);
          WriteInteger(Items[i],sFileVersionMinor,FileVersionMinor);
          WriteInteger(Items[i],sFileVersionRelease,FileVersionRelease);
          WriteInteger(Items[i],sFileVersionBuild,FileVersionBuild);
          WriteString(Items[i],sFileDescription,FileDescription);
          WriteInteger(Items[i],sStatus,SetToWord(@Status));
          end;
        end;
      Free;
      end
    end;
  end;

const
  sKeys = '<VerInfo_Keys>';
  sPlatforms = '<TargetedPlatforms>';
  sVersion = 'FileVersion=';
  sDesc = 'FileDescription=';
  sInternal = 'InternalName*';
  sComp = 'CompanyName=';
  sPAuthor = 'Author=';
  sCopRgt  = 'LegalCopyright=';
  sProdVers = 'ProductVersion=';
  sProduct = 'ProductName=';

function FindKey (ol  : TStringlist; const AKey : string; FirstLine : integer) : integer;
begin
  with ol do for Result:=FirstLine to Count-1 do
    if AnsiContainsText(Strings[Result],AKey) then Exit;
  Result:=-1;
  end;

function GetKey (ol  : TStringlist; const AKey : string) : string;
var
  n,n1,n2 : integer;
begin
  n:=FindKey(ol,AKey,0);
  if n>=0 then begin
    n1:=TextPos(AKey,ol[n]);
    inc(n1,length(AKey));
    n2:=TextPosEx('</',ol[n],n1);
    if n2>0 then Result:=Trim(copy(ol[n],n1,n2-n1)) else Result:='';
    end;
  end;

function GetKeyValue (const VerKey,AKey : string) : string;
var
  n1,n2 : integer;
begin
  Result:='';
  n1:=TextPos(AKey,VerKey);
  if n1>0 then begin
    inc(n1,length(AKey));
    n2:=TextPosEx(';',VerKey,n1);
    if n2=0 then n2:=length(VerKey)+1;
    Result:=Trim(copy(VerKey,n1,n2-n1));
    end;
  end;

// lese die Einstellungen aus den Projektoptionen
procedure TMainForm.ReadProjectOptions (ProjIndex : integer; var pd : TProjectData);
var
  np : integer;
  s,sv : string;
  ol  : TStringlist;
begin
  ol:=TStringList.Create;
  with lbProjects do if ProjIndex>=0 then begin
    s:=AddPath(ExtractFilePath(LastGroupFile),Items[ProjIndex]);
    if FileExists(s) then begin
      ol.LoadFromFile(s);
      sv:=GetKey(ol,SKeys);
      if length(sv)>0 then with pd do begin
        s:=GetKeyValue(sv,sVersion);
        if length(s)>0 then begin
          FileVersionMajor:=ReadNxtInt(s,'.',1);
          FileVersionMinor:=ReadNxtInt(s,'.',1);
          FileVersionRelease:=ReadNxtInt(s,'.',1);
          FileVersionBuild:=ReadNxtInt(s,'.',1);
          end;
        s:=GetKeyValue(sv,sDesc);
        if length(s)=0 then s:=GetKeyValue(sv,sInternal);
        FileDescription:=s;
        end;
      s:=GetKey(ol,sPlatforms);
      if length(s)>0 then with pd do begin
        np:=ReadNxtInt(s,'.',1);
        if np and 1 <>0 then Include(Status,psWin32) else Exclude(Status,psWin32);
        if np and 2 <>0 then Include(Status,psWin64) else Exclude(Status,psWin64);
        end;
      end;
    end;
  ol.Free;
  end;

procedure TMainForm.ReadGlobalOptions;
var
  i,v1,v2 : integer;
  s,sv : string;
  ol  : TStringlist;
begin
  ol:=TStringList.Create;
  v1:=-1; v2:=-1;
  with lbProjects do for i:=0 to Count-1 do begin
    s:=AddPath(ExtractFilePath(LastGroupFile),Items[i]);
    if FileExists(s) then begin
      ol.LoadFromFile(s);
      sv:=GetKey(ol,SKeys);
      if length(sv)>0 then begin
        with edProduct do if length(Text)=0 then Text:=GetKeyValue(sv,sProduct);
        with edCopyright do if length(Text)=0 then Text:=GetKeyValue(sv,sCopRgt);
        with edCompany do if length(Text)=0 then Text:=GetKeyValue(sv,sComp);
        with edAuthor do begin
          if length(Text)=0 then Text:=GetKeyValue(sv,sPAuthor);
          if length(Text)=0 then Text:=GetKeyValue(sv,'Programmierer=');
          end;
        s:=GetKeyValue(sv,sProdVers);
        if v1<0 then v1:=ReadNxtInt(s,'.',1);
        if v2<0 then v2:=ReadNxtInt(s,'.',1);
        end;
      end;
    end;
  if v1>=0 then begin
    udProdMajor.Position:=v1;
    if v2>=0 then udProdMinor.Position:=v2;
    end;
  ol.Free;
  end;

function TMainForm.ReadGroupFile (const GrpName : string) : boolean;
var
  n,k1,k2 : integer;
  s : string;
  GrpFile : TStringList;
begin
  with cbProjectGroup do begin
    n:=Items.IndexOf(GrpName);
    if n>=0 then Items.Delete(n);
    Items.Insert(0,GrpName);
    ItemIndex:=0;
    end;
  LastProjectGrp:=GrpName;
  GrpFile:=TStringList.Create;
  with GrpFile do begin
    LoadFromFile(GrpName);
    with lbProjects do begin
      FreeListObjects(Items); Clear;
      end;
    n:=0;
    while n<Count do begin
      if AnsiStartsText(sKeyProject,Trim(Strings[n])) then begin
        k1:=TextPos(sKeyProject,Strings[n]);
        inc(k1,length(sKeyProject));
        k2:=TextPosEx('>',Strings[n],k1);
        s:=AnsiDequotedStr(copy(Strings[n],k1,k2-k1),Quote);
        if length(s)>0 then lbProjects.AddItem(s,nil);
        end;
      inc(n);
      end;
    Free;
    end;
  Result:=lbProjects.Count>0;
  if Result then begin
    ReadData(NewExt(GrpName,DatExt));
    lbProjects.ItemIndex:=0;
    ShowProjectData;
    end;
  end;

function TMainForm.SelectProjectGroup : boolean;
begin
  with OpenDialog do begin
    if length(cbProjectGroup.Text)>0 then InitialDir:=ExtractFilePath(cbProjectGroup.Text)
    else InitialDir:=LastPath;
    Filename:='';
    Title:=_('Select Delphi project group');
    Filter:=_('Project group files|*.groupproj|all|*.*');
    if Execute then begin
      ReadGroupFile(Filename);
      Result:=true;
      end
    else Result:=false;
    end;
  end;

procedure TMainForm.ShowProjectData;
begin
  with lbProjects do if ItemIndex>=0 then
      with (Items.Objects[ItemIndex] as TProjectInfo).ProjectData do begin
    LastIndex:=ItemIndex;
    udMajor.Position:=FileVersionMajor;
    udMinor.Position:=FileVersionMinor;
    udRelease.Position:=FileVersionRelease;
    udBuild.Position:=FileVersionBuild;
    edProject.Text:=FileDescription;
    cbWin32.Checked:=psWin32 in Status;
    cbWin64.Checked:=psWin64 in Status;
    end;
  end;

procedure TMainForm.SaveProjectData;
begin
  if LastIndex>=0 then with lbProjects do
      with (Items.Objects[LastIndex] as TProjectInfo).ProjectData do begin
    FileVersionMajor:=udMajor.Position;
    FileVersionMinor:=udMinor.Position;
    FileVersionRelease:=udRelease.Position;
    FileVersionBuild:=udBuild.Position;
    FileDescription:=edProject.Text;
    if cbWin32.Checked then Include(Status,psWin32) else Exclude(Status,psWin32);
    if cbWin64.Checked then Include(Status,psWin64) else Exclude(Status,psWin64);
    end;
  end;

procedure TMainForm.sbCheckAllClick(Sender: TObject);
var
  i : integer;
begin
  with lbProjects do for i:=0 to Count-1 do Checked[i]:=true;
  end;

procedure TMainForm.sbCheckNoneClick(Sender: TObject);
var
  i : integer;
begin
  with lbProjects do for i:=0 to Count-1 do Checked[i]:=false;
  end;

procedure TMainForm.lbProjectsClick(Sender: TObject);
begin
  SaveProjectData;
  ShowProjectData;
  end;

procedure TMainForm.lbProjectsClickCheck(Sender: TObject);
begin
  with lbProjects do if ItemIndex>=0 then
      with (Items.Objects[ItemIndex] as TProjectInfo).ProjectData do begin
    if Checked[ItemIndex] then Include(Status,psEnabled) else Exclude(Status,psEnabled);
    end;
  end;

procedure TMainForm.pmiEditListClick(Sender: TObject);
begin
  with cbProjectGroup do EditHistList(Parent.ClientToScreen(Point(Left,Top+Height)),'',Hint,Items);
  end;

procedure TMainForm.btProjectGroupClick(Sender: TObject);
begin
  SelectProjectGroup;
  end;

procedure TMainForm.cbProjectGroupCloseUp(Sender: TObject);
begin
  SaveProjectData;
  WriteData(LastGroupFile);
  with cbProjectGroup do ReadGroupFile(Items[ItemIndex]);
  end;

procedure TMainForm.btTemplateClick(Sender: TObject);
begin
  SelectTemplate;
  end;

procedure TMainForm.btResCompilerClick(Sender: TObject);
begin
  SelectCompiler;
  end;

procedure TMainForm.sbCopyMajorClick(Sender: TObject);
begin
  udMajor.Position:=udProdMajor.Position;
  end;

procedure TMainForm.sbCopyMinorClick(Sender: TObject);
begin
  udMinor.Position:=udProdMinor.Position;
  end;

procedure TMainForm.btBackClick(Sender: TObject);
begin
  ClientHeight:=DefHeight;
  pcMain.ActivePageIndex:=0;
  end;

procedure TMainForm.btSettingsClick(Sender: TObject);
begin
  ClientHeight:=pcMain.Height;
  pcMain.ActivePageIndex:=1;
  end;

procedure TMainForm.btCancelClick(Sender: TObject);
begin
  ClientHeight:=pcMain.Height;
  pcMain.ActivePageIndex:=1;
  end;

procedure TMainForm.btOkClick(Sender: TObject);
begin
  ClientHeight:=pcMain.Height;
  pcMain.ActivePageIndex:=1;
  seTemplate.Lines.SaveToFile(edTemplate.Text);
  end;

procedure TMainForm.btEditClick(Sender: TObject);
begin
  ClientHeight:=pcMain.Height;
  pcMain.ActivePageIndex:=2;
  laTemplateFile.Caption:=edTemplate.Text;
  seTemplate.Lines.LoadFromFile(edTemplate.Text);
  end;

procedure TMainForm.btInfoClick(Sender: TObject);
begin
  InfoDialog(ProgName,Caption+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr);
  end;

procedure TMainForm.btnInsertClick(Sender: TObject);
var
  s : string;
begin
  with cbPlaceholders do if ItemIndex>=0 then begin
    s:='$'+Placeholders[integer(Items.Objects[ItemIndex])].ph;
    seTemplate.SelText:=s;
    end;
  end;

procedure TMainForm.btQuitClick(Sender: TObject);
begin
  Close;
  end;

function TMainForm.SelectTemplate : boolean;
begin
  with OpenDialog do begin
    if length(edTemplate.Text)>0 then InitialDir:=ExtractFilePath(edTemplate.Text)
    else InitialDir:=PrgPath;
    Filename:='';
    Title:=_('Select resource template');
    Filter:=_('Templates|*.'+TemplExt+'|all|*.*');
    Result:=Execute;
    if Result then edTemplate.Text:=Filename;
    end;
  end;

procedure TMainForm.seTemplateStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
var
  s : string;
  n : integer;
begin
  with seTemplate do begin
    n:=RowColToCharIndex(WordStart);
    if (n>0) and (Text[n]='$') then s:=WordAtCursor
    else s:='';
    end;
  if length(s)>0 then begin
    for n:=0 to NumPlaceholders-1 do if AnsiSameText(s,Placeholders[n].ph) then Break;
    if n<NumPlaceholders then cbPlaceholders.ItemIndex:=n;
    end;
  end;

function TMainForm.SelectCompiler : boolean;
begin
  with OpenDialog do begin
    if length(edResCompiler.Text)>0 then InitialDir:=ExtractFilePath(edResCompiler.Text)
    else InitialDir:=ProgPath;
    Filename:=defComp;
    Title:=_('Select resource compiler');
    Filter:=_('Programs|*.exe');
    Result:=Execute;
    if Result then edResCompiler.Text:=Filename;
    end;
  end;

procedure TMainForm.Compile (const APath,RPath,FileName : string);
const
  BUFSIZE = 4096;
  defPipeSize = 64*1024;

var
  si        : TStartupInfo;
  pi        : TProcessInformation;
  saAttr    : TSecurityAttributes;
  hChildStdoutRd,
  hChildStdoutWr  : THandle;
  chBuf           : array [0..BUFSIZE] of AnsiChar;
  dwRead,ec,wc    : DWord;
  s           : string;
  sa          : RawByteString;

  function RawByteToUnicode(sa : RawByteString; CodePage : integer = 1252) : string;
  var
    ta,tu : TBytes;
  begin
    if length(sa)=0 then Result:=''
    else begin
      SetLength(ta,length(sa));
      Move(sa[1],ta[0],Length(ta));
      SetLength(tu,length(sa)*sizeof(Char));
      tu:=TEncoding.Convert(TEncoding.GetEncoding(CodePage),TEncoding.Unicode,ta);
      SetLength(Result,length(sa));
      Move(tu[0],Result[1],Length(tu));
      ta:=nil; tu:=nil;
      end;
    end;

begin
  s:=AnsiQuotedStr(edResCompiler.Text,Quote)+Space
    +AnsiQuotedStr(AddPath(AddPath(APath,RPath),Filename),Quote)
    +' -fo'+AnsiQuotedStr(AddPath(APath,NewExt(Filename,ResExt)),Quote);
  with mmDos,Lines do begin
    Add('=====================================');
    Add(_('Compile resource file: ')+AddPath(RPath,Filename)+' => '+NewExt(Filename,ResExt));
    end;
  Application.ProcessMessages;
  Screen.Cursor:=crHourglass;
// Set the bInheritHandle flag so pipe handles are inherited.
  with saAttr do begin
    nLength:=sizeof(SECURITY_ATTRIBUTES);
    bInheritHandle:=TRUE;
    lpSecurityDescriptor:=nil;
    end;
// Create a pipe for the child process's STDOUT.
  CreatePipe(hChildStdoutRd,hChildStdoutWr,@saAttr,defPipeSize);
  SetHandleInformation(hChildStdoutRd, HANDLE_FLAG_INHERIT,0);
// Create process to start compiler
  FillChar(si, SizeOf(TStartupInfo), 0);
  with si do begin
    cb := Sizeof(TStartupInfo);
    dwFlags := STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;
    wShowWindow:=SW_HIDE;
    hStdOutput:=hChildStdoutWr;
    end;
  try
     if CreateProcess(nil,                // Anwendungsname
                     pchar(s),
                     nil,                // Security
                     nil,                // Security
                     true,               // use InheritHandles
                     NORMAL_PRIORITY_CLASS, // Prioritt
                     nil,                   // Environment
                     nil,                   // Verzeichnis
                     si,pi) then begin
      wc:=WaitForSingleObject(pi.hProcess,10000); //=WAIT_TIMEOUT; // wait 10 s
      GetExitCodeProcess(pi.hProcess,ec); // exit code from called program
      CloseHandle(pi.hProcess);
// Close the write end of the pipe before reading from the
// read end of the pipe.
      if CloseHandle(hChildStdoutWr) then begin
  // Read output from the child process, and write to parent's STDOUT.
        while ReadFile(hChildStdoutRd,chBuf[0],BUFSIZE,dwRead,nil)
              and (dwRead=BUFSIZE) do begin
          sa:=sa+chBuf;
          end;
        if dwRead>0 then begin
          chBuf[dwread]:=#0;
          sa:=sa+chBuf;
          end;
        s:=RawByteToUnicode(sa,850);
        s:=ReplaceStr(s,CrLf,Lf); // Convert Unix style output
        s:=ReplaceStr(s,Lf,CrLf);
        mmDos.SetSelTextBuf(PChar(s));
        end;
      CloseHandle(hChildStdoutRd);
      // DOS-Ausgabe anzeigen
      with mmDos do begin
        SelLength:=0;
        Perform(WM_VSCROLL,SB_BOTTOM,0);
        end;
      if wc<>WAIT_OBJECT_0 then mmDos.Lines.Add('*** '+_('Error: ')+SysErrorMessage(wc));
      end
    else ErrorDialog(_('Error: ')+SysErrorMessage(GetLastError));
  finally
    Screen.Cursor:=crDefault;
    end;
  end;

procedure TMainForm.btProcessClick(Sender: TObject);
var
  sp,s,sr,sc,srp : string;
  i     : integer;
  rsl   : TStringList;
  ok    : boolean;
begin
  SaveProjectData;
  sp:=ExtractFilePath(LastProjectGrp);
  ok:=ForceDirectories(sp);
  srp:=AddPath(sp,edResPath.Text);
  ok:=ForceDirectories(srp);
  if ok then begin
    rsl:=TStringList.Create;
    with lbProjects do for i:=0 to Count-1 do if Checked[i] then
        with (Items.Objects[i] as TProjectInfo).ProjectData do begin
      sr:=DelExt(Items[i]);
      with rsl do begin
        Assign(Template);
        // Project data
        Text:=ReplaceText(Text,'$'+sFileVersionMajor,IntToStr(FileVersionMajor));
        Text:=ReplaceText(Text,'$'+sFileVersionMinor,IntToStr(FileVersionMinor));
        Text:=ReplaceText(Text,'$'+sFileVersionRelease,IntToStr(FileVersionRelease));
        Text:=ReplaceText(Text,'$'+sFileVersionBuild,IntToStr(FileVersionBuild));
        Text:=ReplaceText(Text,'$'+sFileDescription,FileDescription);
        // Group data
        Text:=ReplaceText(Text,'$'+sProductVersionMajor,IntToStr(udProdMajor.Position));
        Text:=ReplaceText(Text,'$'+sProductVersionMinor,IntToStr(udProdMinor.Position));
        Text:=ReplaceText(Text,'$'+sCompanyName,edCompany.Text);
        Text:=ReplaceText(Text,'$'+sLegalCopyright,edCopyright.Text);
        Text:=ReplaceText(Text,'$'+sProductName,edProduct.Text);
        s:=ReplaceText(Text,'$'+sAuthor,edAuthor.Text);
        if psWin32 in Status then begin
          Text:=ReplaceText(s,'$'+sPlatform,sWin32);
          sc:=NewExt(sr+edSuffix32.Text,RcExt);
          try
            SaveToFile(AddPath(srp,sc));
          except
            ok:=false;
            end;
          if ok then Compile(sp,edResPath.Text,sc);
          end;
        if psWin64 in Status then begin
          Text:=ReplaceText(s,'$'+sPlatform,sWin64);
          sc:=NewExt(sr+edSuffix64.Text,RcExt);
          try
            SaveToFile(AddPath(srp,sc));
          except
            ok:=false;
            end;
          if ok then Compile(sp,edResPath.Text,sc);
          end;
        end;
      if not ok then Break;
      end;
    rsl.Free;
    end;
  if not ok then ErrorDialog(Format(_('Could not write to %s'),[sp]));
  end;

end.
