(* Umschalten der Delphi-Entwicklungsumgebung: deutsch - englisch
   nach Sprachen getrennte Projektverwaltung
   verwendbar fr Delphi XE, ..., XE8 und 10

    2011 - 2023, Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)
   
   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

  Hinweise:
  =========
  Ab Delphi XE kann die Sprache der GUI und der Bibliotheken getrennt eingestellt
  werden. Verfgbar sind dabei die bei der Installation ausgewhlten Sprachen
  (z.Zt. englisch, deutsch und franzsisch).

  Fr die Einstellung kann das dem Delphi beiliegende Programm "BDSSetLang.exe"
  verwendet werden. Es muss dazu im interaktiven Modus gestartet werden, d.h. ohne
  Angabe einer Sprache oder mit der Option "/I". Die Sprachen knnen dann
  unabhngig voneinander eingestellt werden.
  Wenn man das Programm mit Angabe einer Sprache (z.B. "BDSSetLang.exe DE") aufruft,
  wird nur die Sprache der Entwicklungsumgebung (nicht die Bibliothekssprache)
  umgeschaltet. In allen Fllen muss die Entwicklungsumgebung danach neu gestartet werden.

  Die vorgenommenen Einstellungen werden in der Registry gespeichert:

  1. GUI-Sprache:
  ---------------
    Key: HKEY_CURRENT_USER\Software\Embarcadero\Locales
    Hier kann fr alle zu den verschiedenen RAD-Studio-Versionen gehrenden Programmen
    die Sprache getrennt angegeben werden (EN, DE, FR oder JA).

  2. Bibliotheks-Sprache:
  -----------------------
    Die Einstellung erfolgt fr jede RAD-Studio-Version getrennt:

    Key: HKEY_CURRENT_USER\Software\Embarcadero\BDS\<version>\MUI
    Value: Translated Library Directory (= EN, DE, FR oder JA)

    Dabei ist
    Rad-Studio      <version>     Compiler-Vers.  Produkt-Vers.  Package-Vers.
    -------------------------------------------------------------------------
    XE              8.0           VER220   22.0   15             150
    XE2             9.0           VER230   23.0   16             160/161
    XE3             10.0          VER240   24.0   17             170
    XE4             11.0          VER250   25.0   18             180
    XE5             12.0          VER260   26.0   19             190
    XE6             14.0          VER270   27.0   20             200
    XE7             15.0          VER280   28.0   21             210
    XE8             16.0          VER290   29.0   22             220
    10 Seattle      17.0          VER300   30.0   23             230
    10.1 Berlin     18.0          VER310   31.0   24             240
    10.2 Tokyo      19.0          VER320   32.0   25             250
    10.3 Rio        20.0          VER330   33.0   26             260
    10.4 Sydney     21.0          VER340   34.0   27             270
    11.0 Alexandria 22.0          VER350   35.0   28             280
    12.0 Athens     23.0          VER360   36.0   29             290


   Vers. 1.0, Sep. 2007
   Vers. 1.3, Nov. 2011 for Delphi XE/XE2
   Vers. 1.4, March 2013 added Delphi XE3, command line option /ini:<ininame>
      can load individual settings for different Delphi versions
   Vers. 2.0, Sept. 2015 supports all installed Delphi versions (XE - 10)
   Vers. 2.1, Jan. 2017 added Delphi 10.1
   Vers. 2.2, Dec. 2017 added Delphi 10.2
   Vers. 2.3, Apr. 2020 added Delphi 10.3
   Vers. 2.4, Jul. 2021 added Delphi 10.4
   Vers. 2.5, Mar. 2022 added Delphi 11.0, changing of GUI language
   Vers. 2.5, Feb. 2023 added Delphi 11.2
   Vers. 2.6, Apr. 2024 added Delphi 12.0

   last modified: April 2024
   *)

unit StartDelphiMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.ExtCtrls, Vcl.ComCtrls, Vcl.ImgList, System.ImageList;

const
  ProgName = 'Start Delphi';
  Vers = ' - Vers. 2.6';
  CopRgt = ' 2011-2024 J. Rathlev, D-24222 Schwentinental';
  EMailAdr = 'kontakt(a)rathlev-home.de';

type
  TDelphiVersion = record
    VName,RegKey : string;
    end;

const
  MaxLang  = 3;
  Languages : array[0..MaxLang-1] of string[2] = ('EN','DE','FR');
  BDS8Key = 'Software\Embarcadero\BDS\8.0\';      // XE   15 VER220
  BDS9Key = 'Software\Embarcadero\BDS\9.0\';      // XE2  16 VER230
  BDS10Key = 'Software\Embarcadero\BDS\10.0\';    // XE3  17 VER240
  BDS11Key = 'Software\Embarcadero\BDS\11.0\';    // XE4  18 VER250
  BDS12Key = 'Software\Embarcadero\BDS\12.0\';    // XE5  19 VER260
  BDS14Key = 'Software\Embarcadero\BDS\14.0\';    // XE6  20 VER270
  BDS15Key = 'Software\Embarcadero\BDS\15.0\';    // XE7  21 VER280
  BDS16Key = 'Software\Embarcadero\BDS\16.0\';    // XE8  22 VER290
  BDS17Key = 'Software\Embarcadero\BDS\17.0\';    // 10   23 VER300
  BDS18Key = 'Software\Embarcadero\BDS\18.0\';    // 10.1 24 VER310
  BDS19Key = 'Software\Embarcadero\BDS\19.0\';    // 10.2 25 VER320
  BDS20Key = 'Software\Embarcadero\BDS\20.0\';    // 10.3 26 VER330
  BDS21Key = 'Software\Embarcadero\BDS\21.0\';    // 10.4 27 VER340
  BDS22Key = 'Software\Embarcadero\BDS\22.0\';    // 11.0 28 VER350
  BDS23Key = 'Software\Embarcadero\BDS\23.0\';    // 12.0 29 VER360
  MuiKey = 'MUI';
  AppKey = 'App';
  KeyName = 'Translated Library Directory';
  LocalesKey = 'Software\Embarcadero\Locales';
  BdsSetLang = 'BdsSetLang.exe';

  FirstVers = 8;
  LastVers  = 23;
  Versions : array [FirstVers..LastVers] of TDelphiVersion = (
             (VName : 'Delphi XE'; RegKey : BDS8Key),
             (VName : 'Delphi XE2'; RegKey : BDS9Key),
             (VName : 'Delphi XE3'; RegKey : BDS10Key),
             (VName : 'Delphi XE4'; RegKey : BDS11Key),
             (VName : 'Delphi XE5'; RegKey : BDS12Key),
             (VName : ''; RegKey : ''),
             (VName : 'Delphi XE6'; RegKey : BDS14Key),
             (VName : 'Delphi XE7'; RegKey : BDS15Key),
             (VName : 'Delphi XE8'; RegKey : BDS16Key),
             (VName : 'Delphi 10';  RegKey : BDS17Key),
             (VName : 'Delphi 10.1'; RegKey : BDS18Key),
             (VName : 'Delphi 10.2'; RegKey : BDS19Key),
             (VName : 'Delphi 10.3'; RegKey : BDS20Key),
             (VName : 'Delphi 10.4'; RegKey : BDS21Key),
             (VName : 'Delphi 11.0'; RegKey : BDS22Key),   // and 11.1
             (VName : 'Delphi 12.0'; RegKey : BDS23Key));

type
  TMainForm = class(TForm)
    btnStartDelphi: TBitBtn;
    btnDelphiPath: TSpeedButton;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    rgLang: TRadioGroup;
    Label2: TLabel;
    btnProject: TSpeedButton;
    OpenDialog: TOpenDialog;
    edProjectDir: TLabeledEdit;
    edDelphiPath: TLabeledEdit;
    btnProjectRoot: TSpeedButton;
    pcMain: TPageControl;
    tsProject: TTabSheet;
    tsSettings: TTabSheet;
    lvProjects: TListView;
    edRegistryKey: TLabeledEdit;
    imgHeader: TImageList;
    btnDelete: TSpeedButton;
    btnDeleteAll: TSpeedButton;
    paSettings: TPanel;
    paProject: TPanel;
    lbVersions: TListBox;
    Label1: TLabel;
    bbSelect: TBitBtn;
    rgIdeLang: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnDelphiPathClick(Sender: TObject);
    procedure btnStartDelphiClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure rgLangClick(Sender: TObject);
    procedure btnProjectClick(Sender: TObject);
    procedure btnProjectRootClick(Sender: TObject);
    procedure lvProjectsColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvProjectsCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure lvProjectsDrawItem(Sender: TCustomListView; Item: TListItem;
      Rect: TRect; State: TOwnerDrawState);
    procedure btnDeleteAllClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure lvProjectsResize(Sender: TObject);
    procedure bbSelectClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ProgName,ProgVers,
    ProgVersDate,
    AppPath,ProgPath,
    UserPath,
    Ininame         : string;
    PrjList         : array[0..MaxLang-1] of TStringList;
    DVersion,
    ColIndex        : integer;
    SortReverse     : boolean;
    procedure GetAppPath (AIndex : integer);
    function GetDelphiPath : boolean;
    procedure ShowProjects(Lang : integer);
  public
    { Public-Deklarationen }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}
{$R Resources\Icons.res}

uses System.Win.Registry, Math, System.DateUtils, System.IniFiles, WinUtils,
  MsgDialogs, StringUtils, NumberUtils, GnuGetText, InitProg, LangUtils,
  ShellDirDlg, PathUtils, WinExecute, ListUtils;

const
  IniExt = 'ini';
  CfgSekt = 'Config';
  EngSekt = 'English';
  GerSekt = 'German';
  LangSekt : array[0..MaxLang-1] of string =('English','German','French');

  iniPath = 'BdsPath';
  iniLeft = 'Left';
  iniTop  = 'Top';
  iniWdt  = 'Width';
  iniHgt  = 'Height';
  iniVer  = 'DelphiVersion';
  iniLang = 'Language';
  iniCount = 'Count';
  iniECnt = 'ECount';
  iniGCnt = 'GCount';
  iniENdx = 'EIndex';
  iniGNdx = 'GIndex';
  iniProj = 'Project';
  iniPRoot = 'ProjectRoot';
  iniIdeLang = 'IdeLanguage';

procedure TMainForm.FormCreate(Sender: TObject);
var
  sp,sv : string;
  n,ne,ng,i,j : integer;
begin
  TranslateComponent (self);
  InitPaths (AppPath,UserPath,ProgPath);
  InitVersion (ProgName,Vers,CopRgt,3,3,ProgName,ProgVers,ProgVersDate);
  for j:=0 to MaxLang-1 do PrjList[j]:=TStringList.Create;
  IniName:=PrgName;
  if ParamCount>0 then for i:=1 to ParamCount do begin
    sp:=ParamStr(i);
    if sp[1]='/' then begin
      Delete(sp,1,1);
      if ReadOptionValue(sp,'ini') then IniName:=sp;
      end
    end;
  IniName:=Erweiter(AppPath,IniName,IniExt);
  with TMemIniFile.Create(IniName) do begin
    edDelphiPath.Text:=ReadString(CfgSekt,IniPath,ProgPath);
    Left:=ReadInteger(CfgSekt,iniLeft,Left);
    Top:=ReadInteger(CfgSekt,iniTop,Top);
    ClientWidth:=ReadInteger(CfgSekt,iniWdt,ClientWidth);
    ClientHeight:=ReadInteger(CfgSekt,iniHgt,ClientHeight);
    sv:=ReadString(CfgSekt,iniVer,'');
    edProjectDir.Text:=ReadString(CfgSekt,iniPRoot,UserPath);
    rgLang.ItemIndex:=ReadInteger(CfgSekt,iniLang,0);
    rgIdeLang.ItemIndex:=ReadInteger(CfgSekt,iniIdeLang,0);
    ne:=ReadInteger(CfgSekt,iniECnt,0);
    ng:=ReadInteger(CfgSekt,iniGCnt,0);
    for j:=0 to MaxLang-1 do begin
      case j of
      1 : n:=ng;  // German use old ini value
      2 : n:=0;   // French new since vers. 2.5
      else n:=ne; // English use old ini value
        end;
      n:=ReadInteger(LangSekt[j],iniCount,n);   // new project count
      for i:=0 to n-1 do begin                  // read projects
        sp:=ReadString(LangSekt[j],iniProj+ZStrint(i,2),'');
        if (length(sp)>0) then begin
          sp:=MakeAbsolutePath(edProjectDir.Text,sp);
          if FileExists(sp) then PrjList[j].Add(sp);
          end;
        end;
      end;
    Free;
    end;
  lbVersions.Clear; DVersion:=-1;
  // Retrieve keys to RAD Studio
  with TRegistry.Create(KEY_READ) do begin
    RootKey:=HKEY_LOCAL_MACHINE; //HKEY_CURRENT_USER;
    for i:=FirstVers to LastVers do with Versions[i] do if (length(RegKey)>0) then begin
      try
        if (OpenKey(RegKey,false)) then begin
          n:=lbVersions.Items.AddObject(VName,TObject(i));
          if AnsiSameText(VName,sv) then begin
            DVersion:=i;
            lbVersions.ItemIndex:=n;
            end;
          end;
      finally
        CloseKey;
        end;
      end;
    Free;
    end;
  end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  i,j : integer;
begin
  with TMemIniFile.Create(IniName) do begin
    Clear;
    WriteString(CfgSekt,IniPath,edDelphiPath.Text);
    WriteInteger(CfgSekt,iniLeft,Left);
    WriteInteger(CfgSekt,iniTop,Top);
    WriteInteger(CfgSekt,iniWdt,ClientWidth);
    WriteInteger(CfgSekt,iniHgt,ClientHeight);
    WriteString(CfgSekt,iniVer,Versions[DVersion].VName);
    WriteString(CfgSekt,iniPRoot,edProjectDir.Text);
    WriteInteger(CfgSekt,iniLang,rgLang.ItemIndex);
    WriteInteger(CfgSekt,iniIdeLang,rgIdeLang.ItemIndex);
    DeleteKey(CfgSekt,iniECnt);    // from former version
    DeleteKey(CfgSekt,iniGCnt);
    for j:=0 to MaxLang-1 do with PrjList[j] do begin
      EraseSection(LangSekt[j]);
      WriteInteger(LangSekt[j],iniCount,Count);   // new project count
      for i:=0 to Count-1 do WriteString(LangSekt[j],iniProj+ZStrint(i,2),Strings[i]);
      end;
    UpdateFile;
    Free;
    end;
  for i:=0 to MaxLang-1 do PrjList[i].Free;
  ColIndex:=0; SortReverse:=false;
  with lvProjects do begin
    for i:=0 to Columns.Count-1 do Columns[i].ImageIndex:=-1;
    Columns[0].ImageIndex:=0;
    end;
  end;

procedure TMainForm.GetAppPath (AIndex : integer);
begin
  with TRegistry.Create(KEY_READ),Versions[AIndex] do begin
    RootKey:=HKEY_CURRENT_USER;
    try
      if OpenKey(RegKey,false) then begin
        edDelphiPath.Text:=ReadString(AppKey);
        edRegistryKey.Text:=RegKey;
        end;
    finally
      CloseKey;
      end;
    Free;
    end;
  Caption:=_('Start ')+Versions[AIndex].VName+' '+ProgVers;
  end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if lbVersions.Count=0 then begin
    ErrorDialog(_('No supported Delphi IDE found on your system!'));
    Close;
    end
  else if DVersion<0 then begin    // select
    with lbVersions do begin
      ItemIndex:=Items.Count-1; // select newest
      DVersion:=integer(Items.Objects[ItemIndex]);
      end;
    GetAppPath(DVersion);
    pcMain.ActivePage:=tsSettings;
    end
  else begin
    GetAppPath(DVersion);
    pcMain.ActivePage:=tsProject;
    end;
//  if not FileExists(edDelphiPath.Text) and not GetDelphiPath then Close;
  if not DirectoryExists(edProjectDir.Text) then btnProjectRootClick(Sender);
  ShowProjects(rgLang.ItemIndex);
  end;

procedure TMainForm.bbSelectClick(Sender: TObject);
begin
  with lbVersions do DVersion:=integer(Items.Objects[ItemIndex]);
  GetAppPath(DVersion);
  end;


procedure TMainForm.ShowProjects(Lang : integer);
var
  sl : TStringList;
  i  : integer;
  s  : string;

  function GetFileDateTime (const FileName : string) : TDateTime;
  begin
    if not FileAge(Filename,Result) then Result:=0;
    end;

begin
  if not Visible then Exit;
  sl:=PrjList[Lang];
  with lvProjects do begin
    Items.BeginUpdate;
    Clear;
    end;
  with sl do begin
    for i:=0 to Count-1 do begin
      s:=Strings[i];
      with lvProjects.Items.Add do begin
        Caption:=IntToStr(i+1);
        Data:=pointer(i);
        SubItems.Add(ExtractFilename(s));
        SubItems.Add(MakeRelativePath(edProjectDir.Text,s));
        SubItems.Add(DateTimeTostr(GetFileDateTime(s)));
        ImageIndex:=-1;
        end
      end;
    end;
  with lvProjects do begin
    if Items.Count>0 then ItemIndex:=0;
    Items.EndUpdate;
    AlphaSort;
    if assigned(Selected) then Selected.MakeVisible(false);
    end;
  end;

procedure TMainForm.lvProjectsColumnClick(Sender: TObject; Column: TListColumn);
var
  i : integer;
begin
  with lvProjects do for i:=0 to Columns.Count-1 do Columns[i].ImageIndex:=-1;
  if Column.Index=ColIndex then SortReverse:=not SortReverse else SortReverse:=false;
  with Column do if SortReverse then ImageIndex:=1 else ImageIndex:=0;
  ColIndex:=Column.Index;
  with lvProjects do begin
    AlphaSort;
    Selected.MakeVisible(false);
    end;
  end;

procedure TMainForm.lvProjectsCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
var
  dt1,dt2 : TDateTime;
  n1,n2   : integer;
begin
  if ColIndex=0 then begin
    if not TryStrToInt(Item1.Caption,n1) then n1:=0;
    if not TryStrToInt(Item2.Caption,n2) then n2:=0;
    Compare:=CompareValue(n1,n2);
    end
  else if (ColIndex<=2) and (Item1.SubItems.Count>0) then // Name, Path
    Compare:=CompareText(Item1.SubItems[ColIndex-1],Item2.SubItems[ColIndex-1])
  else if (Item1.SubItems.Count>1) then begin  // Date
    if not TryStrToDateTime(Item1.SubItems[2],dt1) then dt1:=0;
    if not TryStrToDateTime(Item2.SubItems[2],dt2) then dt2:=0;
    Compare:=CompareDateTime(dt1,dt2);
    end;
  if SortReverse then Compare:=-Compare;
  end;

procedure TMainForm.lvProjectsDrawItem(Sender: TCustomListView; Item: TListItem;
  Rect: TRect; State: TOwnerDrawState);
var
  i,x,w,y : integer;

  function StripString (s : string; Canvas: TCanvas; MaxLen: Integer) : string;
  begin
    if (length(s)>0) and (Canvas.TextWidth(s)>MaxLen) then begin
      while Canvas.TextWidth(s+'...')>MaxLen do Delete(s,length(s),1);
      Result:=s+'...';
      end
    else Result:=s;
    end;

begin
  with lvProjects,Canvas do begin
    if odSelected in State then begin
      if lvProjects=ActiveControl then Brush.Color:=$00AFAFFF else Brush.Color:=clSkyBlue;
      end
    else Brush.Color:=clWhite;
    with Rect do begin
      x:=Left+1; y:=Top+1;
      end;
    FillRect(Rect);
    w:=Columns[0].Width;
    TextOut(x+2,y,Item.Caption);
    x:=w;
    with Item.SubItems do for i:=0 to Count-1 do begin
      w:=Columns[i+1].Width;
      if i=0 then Font.Style:=[fsBold] else Font.Style:=[];
      TextOut(x,y,StripString(Strings[i],Canvas,w-6));
      x:=x+w;
      end;
    end;
  end;

procedure TMainForm.lvProjectsResize(Sender: TObject);
var
  w : integer;
begin
  with lvProjects do begin
    w:=(Width-171) div 2;
    Columns[1].Width:=w; Columns[2].Width:=w;
    end;
  end;

procedure TMainForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog(ProgName,ProgName+ProgVers+' - '+ProgVersDate+sLineBreak+CopRgt
           +sLineBreak+'E-Mail: '+EmailAdr);
  end;

procedure TMainForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

function TMainForm.GetDelphiPath : boolean;
begin
  with OpenDialog do begin
    Title:=_('Search for Delphi IDE');
    if length(edDelphiPath.Text)>0 then InitialDir:=ExtractFilePath(edDelphiPath.Text)
    else InitialDir:=ProgPath;
    DefaultExt:='';
    Filter:=_('Executable programs|*.exe|all|*.*');
    Filename:='bds.exe';
    Result:=Execute;
    if Result then edDelphiPath.Text:=Filename;
    end;
  end;

procedure TMainForm.btnDeleteAllClick(Sender: TObject);
begin
  if ConfirmDialog(_('Remove all project entries for the selected language?')) then begin
    PrjList[rgLang.ItemIndex].Clear;
    ShowProjects(rgLang.ItemIndex);
    end;
  end;

procedure TMainForm.btnDeleteClick(Sender: TObject);
begin
  if assigned(lvProjects.Selected) and ConfirmDialog(_('Remove selected project entry from list?')) then begin
    PrjList[rgLang.ItemIndex].Delete(integer(lvProjects.Selected.Data));
    ShowProjects(rgLang.ItemIndex);
    end;
  end;

procedure TMainForm.btnDelphiPathClick(Sender: TObject);
begin
  GetDelphiPath;
  end;

procedure TMainForm.btnProjectRootClick(Sender: TObject);
var
  s : string;
begin
  s:=edProjectDir.Text;
  if length(s)=0 then s:=UserPath;
  if ShellDirDialog.Execute (_('Delphi project directory'),true,true,false,'',s) then begin
    edProjectDir.Text:=IncludeTrailingPathDelimiter(s);
    end;
  end;

procedure TMainForm.rgLangClick(Sender: TObject);
begin
  ShowProjects(rgLang.ItemIndex);
  end;

procedure TMainForm.btnProjectClick(Sender: TObject);
begin
  with OpenDialog do begin
    Title:=_('Select Delphi project');
    InitialDir:=edProjectDir.Text;
    DefaultExt:='';
    Filter:=_('Delphi projects|*.groupproj;*.dproj;*.dpr;*.dpk|all|*.*');
    Filename:='';
    if Execute then begin
      AddToHistory(PrjList[rgLang.ItemIndex],Filename);
      ShowProjects(rgLang.ItemIndex);
      end;
    end;
  end;

procedure TMainForm.btnStartDelphiClick(Sender: TObject);
var
  sp,si : string;
  n  : integer;
  ok : boolean;
  ph : THandle;
begin
  with lvProjects do if assigned(Selected) then begin
    n:=integer(Selected.Data);
    sp:=PrjList[rgLang.ItemIndex][n];
    AddToHistory(PrjList[rgLang.ItemIndex],sp);
    ShowProjects(rgLang.ItemIndex);
//  SetCurrentDir(edDelphiPath.Text);
    ok:=true;
    with TRegistry.Create do begin
      RootKey:=HKEY_CURRENT_USER;
      try
        if OpenKey(LocalesKey,false) then si:=ReadString(edDelphiPath.Text);
      finally
        CloseKey;
        end;
      if not AnsiSameText(si,Languages[rgIdeLang.ItemIndex]) then begin  // change IDE language
        ExecuteProcess(AnsiQuotedStr(ExtractFilePath(edDelphiPath.Text)+BdsSetLang,Quote)+' '+
          Languages[rgIdeLang.ItemIndex],'',[pfShowConsole],1000);
        end;
      try
        if OpenKey(SetDirName(edRegistryKey.Text)+MuiKey,true) then  // change lib language
          WriteString(KeyName,Languages[rgLang.ItemIndex])
        else ok:=false;
      finally
        CloseKey;
        end;
      Free;
      end;
    if ok then begin
      try
        ph:=StartProcess(AnsiQuotedStr(edDelphiPath.Text,Quote)+' '+
           AnsiQuotedStr(sp,Quote),edProjectDir.Text);
        CloseHandle(ph);
        Close;
      except
        ErrorDialog(Format(_('Could not start'+sLineBreak+'%s!'),[edDelphiPath.Text]))
        end;
      end
    else ErrorDialog(_('Could not change MUI registry key!'));
    end;
  end;

end.
